{$UNDEF LOGFILE}

unit Main;

{$MODE Delphi}

interface

uses
  Windows, LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, StdCtrls, Buttons;

function GetProcessImageFileName(hProcess: HANDLE; lpFilename: LPTSTR; nSize: DWORD): DWORD;
stdcall; external 'psapi.dll' name 'GetProcessImageFileNameA';

type

  { TMainForm }

  TMainForm = class(TForm)
    GrabImage: TImage;
    MainMenu1: TMainMenu;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure GrabImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GrabImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GrabImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
    ModuleFileName: String;
    ClassName: String;
    WindowTitle: String;
    SelX, SelY, SelX1, SelY1 : Integer;
    Done: Boolean;
{$IFDEF LOGFILE}
    LogFile: TextFile;
{$ENDIF}
    procedure Grab(bm: TBitMap);
  end;
  CGException = class(Exception);

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

procedure MyMessageBox(Message: string; Title: string = 'Information');
begin
  Application.MessageBox(PChar(Message), PChar(Title), MB_TOPMOST);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  SelX := 0;
  SelY := 0;
  SelX1 := 0;
  SelY1 := 0;
  MainForm.DoubleBuffered := true;
  if ParamCount = 0 then
  begin
    MyMessageBox('Usage: ' + ExtractFileName(Application.ExeName) + ' file-name');
    Application.Terminate;
  end;
  MainForm.BringToFront;
  MyMessageBox('Press <F7> to grab a region.' + LineEnding +
    'Press <F8> to close ClientGrab and continue with the script.' + LineEnding +
    'Press <F9> to exit ClientGrab and exit the script.' + LineEnding +
    'The grabbed region will be saved to: ' + LineEnding +
    ParamStr(1));
  GrabImage.Hint := ParamStr(1);
  Done := False;
{$IFDEF LOGFILE}
  AssignFile(LogFile, 'clientgrab.log');
  Rewrite(LogFile);
{$ENDIF}
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);

  procedure FlashForm;
  var
    iFlash: Integer;
  begin
    for iFlash := 0 to 5 do
    begin
    if odd(iFlash) then
      GrabImage.Show
    else
      GrabImage.Hide;
      Application.ProcessMessages;
      Sleep(100);
    end;
  end;

begin
  if Key = VK_F7 then
  begin
    MyMessageBox(
      'ClientGrab will hide itself now.' + LineEnding +
      'Please activate the target window within 5 seconds and wait.' + LineEnding +
      'Make sure you click in an unused part of the window to avoid changes in it.');
    MainForm.Hide;
    Sleep(5000);
    Grab(GrabImage.Picture.Bitmap);
    MainForm.FormStyle := fsStayOnTop;
    MainForm.Show;
    MainForm.BringToFront;
    MyMessageBox(
      'Now select the region to grab.');
  end
  else if Key = VK_F8 then
  begin
    if Done then
      MainForm.Close
    else
      MyMessageBox('Please select a region.');
  end
  else if Key = VK_F9 then
  begin
    ExitCode := 1;
    Application.Terminate;
  end;
end;

procedure TMainForm.Grab(bm: TBitMap);
var
  hWnd: THandle;
  pPid: DWORD;
  hProcess: THandle;
  n: Integer;
  hdcSrc: THandle;
  SourceRect: TRect;
  //bmp: TBitmap;
  //cBmp: HBITMAP;
  TopLeft: TPoint;
begin
  hWnd := GetForeGroundWindow;
  if hWnd = 0 then Exit;
  try
    //RedrawWindow(hWnd, nil, 0, RDW_INVALIDATE or RDW_UPDATENOW);
    //Application.ProcessMessages;

    GetWindowThreadProcessID(hWnd, pPid);
    hProcess := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, False, pPid);
    SetLength(ModuleFileName, 255);
    n := GetProcessImageFileName(hProcess, PChar(ModuleFileName), 255);
    //On Windows 2003 GetProcessImageFileName always returns 129?
    n := Strlen(PChar(ModuleFileName));
    SetLength(ModuleFileName, n);

    SetLength(ClassName, 255);
    n := GetClassName(hWnd, PChar(ClassName), 255);
    SetLength(ClassName, n);

    SetLength(WindowTitle, 255 );
    n := SendMessage(hWnd, WM_GETTEXT, Length(WindowTitle), Integer(WindowTitle));
    SetLength(WindowTitle, n);
{$IFDEF LOGFILE}
    writeln(LogFile, 'ClassName=', ClassName);
    writeln(LogFile, 'ModuleFileName=', ModuleFileName);
    writeln(LogFile, 'WindowTitle=', WindowTitle);
{$ENDIF}

    hdcSrc := GetDC(GetDesktopWindow);
    if hdcSrc = 0 then
      raise(CGException.Create('hdcSrc returned nil'));

    Windows.GetClientRect(hWnd, SourceRect);
{$IFDEF LOGFILE}
    writeln(LogFile, 'SourceRect Top/Left/Bottom/Right=', SourceRect.Top, ' ', SourceRect.Left, ' ',
      SourceRect.Bottom, ' ', SourceRect.Right);
{$ENDIF}
    bm.Width  := SourceRect.Right - SourceRect.Left;
    bm.Height := SourceRect.Bottom - SourceRect.Top;

    TopLeft.X := SourceRect.Left;
    TopLeft.Y := SourceRect.Top;
    Windows.ClientToScreen(hWnd, TopLeft);
    {$IFDEF LOGFILE}
    writeln(LogFile, 'TopLeft X/Y=', TopLeft.X, ' ', TopLeft.Y);
    {$ENDIF}

    //bmp := Graphics.TBitmap.Create;
    //cBmp := CreateCompatibleBitmap(hdcSrc, bm.Width, bm.Height);
    //bmp.Handle := cBmp;
    //BitBlt(bmp.Canvas.Handle, 0, 0, bm.Width, bm.Height, hdcSrc, SourceRect.Left, SourceRect.Top, SRCCOPY);
    //bmp.Canvas.Changed;

    //StretchBlt(bm.Canvas.Handle, 0, 0, bm.Width, bm.Height, hdcSrc,
      //0, 0, SourceRect.Right - SourceRect.Left, SourceRect.Bottom - SourceRect.Top,
      //SRCCOPY);
    BitBlt(bm.Canvas.Handle, 0, 0, bm.Width, bm.Height, hdcSrc, TopLeft.X, TopLeft.Y, SRCCOPY);
    //BitBlt(bm.Canvas.Handle, 0, 0, bm.Width, bm.Height, bmp.Canvas.Handle, SourceRect.Left, SourceRect.Top, SRCCOPY);
    bm.Canvas.Changed;
  finally
    ReleaseDC(GetDesktopWindow, hdcSrc);
  end;
end;

procedure TMainForm.GrabImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  SelX := X;
  SelY := Y;
  SelX1 := X;
  SelY1 := Y;
  with GrabImage.Canvas do
  begin
    Pen.Width := 1;
    Pen.Style := psDashDotDot;
    Pen.Mode := pmNotXOR;
    Brush.Style := bsClear;
    Pen.Color := clBlue;
  end;
end;

procedure TMainForm.GrabImageMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if ssLeft in Shift then
  begin
    GrabImage.Canvas.Rectangle(SelX, SelY, SelX1, SelY1);
    SelX1 := X;
    SelY1 := Y;
    GrabImage.Canvas.Rectangle(SelX, SelY, SelX1, SelY1);
  end;
end;

procedure TMainForm.GrabImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  TheFile: TextFile;
begin
  GrabImage.Canvas.Rectangle(SelX, SelY, SelX1, SelY1);
  if X <= SelX then
  begin
    SelX1 := SelX;
    SelX := X;
  end
  else
    SelX1 := X;

  if Y <= SelY then
  begin
    SelY1 := SelY;
    SelY := Y;
  end
  else
    SelY1 := Y;

  if (SelX1 > SelX) and (SelY1 > SelY) and (ParamCount > 0) then
  begin
    WindowTitle := InputBox('Edit Window Title',
      'Executable: ' + ExtractFileName(ModuleFileName) + LineEnding +
      'Window class: ' + ClassName + LineEnding + LineEnding +
      'You may shorten the window title FROM THE RIGHT to remove parts that might change' + LineEnding +
      ' like usernames and version numbers. This will also speed up finding the window.' + LineEnding +
      'You can also use the keyword FOCUS to dynamically get the window title of the window with input focus.' + LineEnding +
      'This is REQUIRED if the Window class contains elements that might change.', WindowTitle);
    AssignFile(TheFile, ParamStr(1));
    ReWrite(TheFile);
    if WindowTitle = 'FOCUS' then
      Writeln(TheFile, WindowTitle)
    else
      Writeln(TheFile, ExtractFileName(ModuleFileName) + '|' + ClassName + '|' + WindowTitle);
    Writeln(TheFile, 1);
    Writeln(TheFile, SelX);
    Writeln(TheFile, SelY);
    Writeln(TheFile, SelX1 - SelX);
    Writeln(TheFile, SelY1 - SelY);
    CloseFile(TheFile);
    Done := True;
    MyMessageBox(
      'Created file ' + ParamStr(1) + LineEnding + LineEnding +
      'If you are not satisfied with the region you can select another region.' + LineEnding +
      'Press <F8> to close ClientGrab and continue with the script.' + LineEnding +
      'Press <F9> to exit ClientGrab and exit the script.');
  end;
end;

procedure TMainForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  if Done then
  begin
{$IFDEF LOGFILE}
    CloseFile(LogFile);
{$ENDIF}
    CloseAction := caFree;
  end
  else
  begin
    MyMessageBox('Please select a region.');
    CloseAction := caNone;
  end;
end;

end.
